class Function < ActiveRecord::Base
  belongs_to :user
  belongs_to :group
  has_many :function_outputs, :order => :position, :dependent => :destroy
  has_many :function_arguments, :order => :position, :dependent => :destroy

  validates_presence_of :name, :input_num, :script

  def self.find_for_user(user, *args)
    args = [:all] if args.length == 0
    if args.length == 1
      args.push( {:conditions => add_user_conditions(user)} )
    elsif args[-1].is_a?(Hash)
      args[-1][:conditions] = add_user_conditions(user,args[-1][:conditions])
    else
      # args must be IDs --> put them in an array
      args.each do |x|
        if !x.is_a?(Integer)
          raise ArgumentError,
            "ID must be integers, but #{x.inspect} isn't. Or you need to give"+
            "a Hash as the 2nd argument, which is now #{args[1].inspect}."
        end
      end
      args = [ args, {:conditions => add_user_conditions(user)} ]
    end
    return Function.find(*args)
  end

  def self.count_for_user(user, *args)
    args[0] = add_user_conditions(user,args[0])
    return Function.count(*args)
  end

  def get_script
    result = ""
    function_arguments.length.times{|i|
      arg = function_arguments[i]
      result += " arg#{i} = arg#{i}.#{to_type(arg.name)}\n"
    }
    result += script
    return result
  end

  protected
  def self.add_user_conditions(user, cond=nil)
    grp_cond = Group.group_conditions(user)
    if cond
      if Array === cond
        new_conditions = ['( '+ cond[0] +' ) AND ' + grp_cond ] + cond[1..-1]
      else
        new_conditions = '( '+ cond +' ) AND ' + grp_cond
      end
    else
      new_conditions = grp_cond
    end
    return new_conditions
  end

  def to_type(type)
    case type.type
    when "int"
      "to_i"
    when "float"
      "to_f"
    when "string"
      "to_s"
    else
      raise "argument type of function is invalid"
    end
  end

end
