class Group < ActiveRecord::Base
  belongs_to :owner, :class_name => "User", :foreign_key => "user_id"
  has_many :members, :through => :group_members, :source => :user, :order => :login
  has_many :group_members, :dependent => :destroy

  validates_presence_of :name
  validates_uniqueness_of :name, :on => :create

  def self.find_for_user(user)
    return [] unless user
    return user.belonging_groups
  end

  # Rutern SQL conditions to limit search results to the groups for the user
  def self.group_conditions(user, cname=nil)
    groups = find_for_user(user)
    cname += "." if cname
    if groups.length != 0
      grp_joined = groups.collect{|x| x.id}.join(',')
      grp_cond = " ( #{cname}group_id IS NULL OR #{cname}group_id IN (#{grp_joined}) ) "
    else
      grp_cond = " (#{cname}group_id IS NULL) "
    end
  end

end
