require File.dirname(__FILE__) + '/../test_helper'
require File.expand_path(File.dirname(__FILE__) + "/../../lib/narray_yaml")

class DirectoryTest < Test::Unit::TestCase
  fixtures :directories, :variables, :keyword_attributes, :users, :groups,
           :group_members

  def setup
    @dir = Directory.find(2)
  end

  def test_create
    reanalysis = directories(:reanalysis)
    assert_kind_of Directory, @dir
    assert_equal reanalysis.id, @dir.id
    assert_equal reanalysis.name, @dir.name
    assert_equal reanalysis.parent_id, @dir.parent_id
    assert_equal reanalysis.plain_file, @dir.plain_file
  end

  def test_update
    name = "changed_name"
    @dir.name = name
    assert @dir.save, @dir.errors.full_messages.join("; ")
    @dir.reload
    assert_equal name, @dir.name
  end

  def test_children_for_user
    dir = Directory.find_by_path('/reanalysis/ncep')
    children = dir.children_for_user( User.find_by_login('bob') )
    assert_equal children.collect{|x| x.path}.sort, 
                 ["/reanalysis/ncep/T.jan.nc", "/reanalysis/ncep/UV.jan.nc"]
    children = dir.children_for_user( User.find_by_login('longbob') )
    assert_equal children.collect{|x| x.path}.sort, 
                 ["/reanalysis/ncep/T.jan.nc"]
  end

  def test_children
    ncep_jan = directories(:ncep_t_jan)
    children = @dir.children
    assert_kind_of Array, children
    assert_equal 2, children.length
    assert_kind_of Directory, children[1]
    ncep = children.find(:first, :conditions => 'name = "ncep"')
    assert_kind_of Directory, ncep
    assert_kind_of Directory, ncep.children[0]
    assert_equal ncep_jan.name, children[1].children[0].name
  end

  def test_variables
    ncep = @dir.children.find(:first, :conditions => 'name = "ncep"')
    ncep_jan = ncep.children.find(:first, :conditions => 'name = "T.jan.nc"')
    assert_kind_of Directory, ncep_jan
    variables = ncep_jan.variables_for_user(nil)
    assert_kind_of Array, variables
    assert_kind_of Variable, variables[0]
    size = @dir.variables_for_user(nil).size
    name = "test_dir#{@dir.id}"
    path = "/dummy/path"
    Variable.new(:name => name, :path => path, :directory_id => @dir.id, :gphysizable => false).save
    assert_equal size+1, @dir.variables_for_user(nil).size
    var = Variable.find(:first, :conditions => ['name = ? and directory_id = ?', name, @dir.id])
    assert_kind_of Variable, var
    var = Variable.find(:first, :conditions => ['name = ?', name])
    assert_kind_of Variable, var
    assert_equal @dir.id, var.directory_id
  end

  def test_keyword_attributes
    assert_kind_of Array, @dir.keyword_attributes
    assert_kind_of KeywordAttribute, @dir.keyword_attributes[0]
    size = @dir.keyword_attributes.size
    name = "test"
    value = "this is test attribute"
    @dir.keyword_attributes.create(:name => name, :value => value)
    assert_equal size+1, @dir.keyword_attributes.size
    ka = @dir.keyword_attributes.find(:first, :conditions => ['name = ? and value = ?', name, value])
    assert_kind_of KeywordAttribute, ka
    ka = KeywordAttribute.find(:first, :conditions => ['name = ? and value = ?', name, value])
    assert_kind_of KeywordAttribute, ka
    @dir.keyword_attributes.delete(ka)
    assert_equal nil, KeywordAttribute.find(:first, :conditions => ['name = ? and value = ?', name, value])
    name = 'new'
    dir = Directory.new(:name => name, :parent_id => @dir.id)
    assert dir.save, dir.errors.full_messages.join("; ")
    assert_kind_of Directory, @dir.children.find(:first, :conditions => ['name = ?', name])
    name = "test_dir#{dir.id}"
    kw = KeywordAttribute.new(:name => name, :directory_id => dir.id )
    assert kw.save, kw.errors.full_messages.join("; ")
    assert_kind_of KeywordAttribute, dir.keyword_attributes[0]
    assert_equal 1, KeywordAttribute.find(:all, :conditions => ['name = ?', name]).size
    dir.destroy
    assert_equal 0, KeywordAttribute.find(:all, :conditions => ['name = ?', name]).size
  end

end
